import { Observer } from '../../runtime/observer.js';
import { defineConfig, copy, equal, removeProperties, generateId } from '@agile-ts/utils';
import { RuntimeJob } from '../../runtime/runtime.job.js';

class GroupObserver extends Observer {
  constructor(group, config = {}) {
    super(group.agileInstance(), defineConfig(config, { value: group._output }));
    this.group = () => group;
    this.nextGroupOutput = copy(group._output);
  }
  ingest(config = {}) {
    this.ingestOutput(this.group().nextGroupOutput, config);
  }
  ingestOutput(newGroupOutput, config = {}) {
    const group = this.group();
    config = defineConfig(config, {
      perform: true,
      force: false
    });
    if (group.isPlaceholder) {
      config.force = true;
    }
    this.nextGroupOutput = copy(newGroupOutput);
    if (equal(group._output, this.nextGroupOutput) && !config.force)
      return;
    const job = new RuntimeJob(this, {
      ...removeProperties(config, ["perform"]),
      ...{
        key: config.key ?? `${this._key != null ? this._key + "_" : ""}${generateId()}_output`
      }
    });
    this.agileInstance().runtime.ingest(job, {
      perform: config.perform
    });
  }
  perform(job) {
    const observer = job.observer;
    const group = observer.group();
    group._output = copy(observer.nextGroupOutput);
    group.nextGroupOutput = copy(observer.nextGroupOutput);
    job.observer.previousValue = copy(job.observer.value);
    job.observer.value = copy(group._output);
  }
}

export { GroupObserver };
