import { extractRelevantObservers } from '../utils.js';
import { State } from '../state/state.js';
import { defineConfig, isAsyncFunction, removeProperties } from '@agile-ts/utils';
import { ComputedTracker } from './computed.tracker.js';
import { LogCodeManager } from '../logCodeManager.js';

class Computed extends State {
  constructor(agileInstance, computeFunction, config = {}) {
    super(agileInstance, null, {
      key: config.key,
      dependents: config.dependents
    });
    this.deps = new Set();
    this.hardCodedDeps = [];
    this.isComputed = true;
    config = defineConfig(config, {
      computedDeps: [],
      autodetect: !isAsyncFunction(computeFunction)
    });
    this.agileInstance = () => agileInstance;
    this.computeFunction = computeFunction;
    this.config = {
      autodetect: config.autodetect
    };
    this.hardCodedDeps = extractRelevantObservers(config.computedDeps).filter((dep) => dep !== void 0);
    this.deps = new Set(this.hardCodedDeps);
    this.deps.forEach((observer) => {
      observer.addDependent(this.observers["value"]);
    });
    this.recompute({ autodetect: config.autodetect, overwrite: true });
  }
  recompute(config = {}) {
    config = defineConfig(config, {
      autodetect: false
    });
    this.compute({ autodetect: config.autodetect }).then((result) => {
      this.observers["value"].ingestValue(result, removeProperties(config, ["autodetect"]));
    });
    return this;
  }
  updateComputeFunction(computeFunction, deps = [], config = {}) {
    config = defineConfig(config, {
      autodetect: this.config.autodetect
    });
    this.deps.forEach((observer) => {
      observer.removeDependent(this.observers["value"]);
    });
    this.hardCodedDeps = extractRelevantObservers(deps).filter((dep) => dep !== void 0);
    this.deps = new Set(this.hardCodedDeps);
    this.deps.forEach((observer) => {
      observer.addDependent(this.observers["value"]);
    });
    this.computeFunction = computeFunction;
    this.recompute(removeProperties(config, ["overwriteDeps"]));
    return this;
  }
  async compute(config = {}) {
    config = defineConfig(config, {
      autodetect: this.config.autodetect
    });
    if (config.autodetect)
      ComputedTracker.track();
    const computedValue = this.computeFunction();
    if (config.autodetect) {
      const foundDeps = ComputedTracker.getTrackedObservers();
      this.deps.forEach((observer) => {
        if (!foundDeps.includes(observer) && !this.hardCodedDeps.includes(observer)) {
          this.deps.delete(observer);
          observer.removeDependent(this.observers["value"]);
        }
      });
      foundDeps.forEach((observer) => {
        if (!this.deps.has(observer)) {
          this.deps.add(observer);
          observer.addDependent(this.observers["value"]);
        }
      });
    }
    return computedValue;
  }
  persist() {
    LogCodeManager.log("19:03:00");
    return this;
  }
}

export { Computed };
