import { Integration } from './integration.js';
import { defineConfig } from '@agile-ts/utils';
import { LogCodeManager } from '../logCodeManager.js';

const onRegisterInitialIntegrationCallbacks = [];
const _Integrations = class {
  constructor(agileInstance, config = {}) {
    this.integrations = new Set();
    config = defineConfig(config, {
      autoIntegrate: true
    });
    this.agileInstance = () => agileInstance;
    if (config.autoIntegrate) {
      _Integrations.onRegisterInitialIntegration((integration) => {
        this.integrate(integration);
      });
    }
  }
  static addInitialIntegration(integration) {
    if (integration instanceof Integration) {
      onRegisterInitialIntegrationCallbacks.forEach((callback) => callback(integration));
      _Integrations.initialIntegrations.push(integration);
    }
  }
  static onRegisterInitialIntegration(callback) {
    onRegisterInitialIntegrationCallbacks.push(callback);
    _Integrations.initialIntegrations.forEach((integration) => {
      callback(integration);
    });
  }
  async integrate(integration) {
    if (integration._key == null) {
      LogCodeManager.log("18:03:00", [integration._key, this.agileInstance().key], integration);
      return false;
    }
    if (integration.methods.bind)
      integration.ready = await integration.methods.bind(this.agileInstance());
    else
      integration.ready = true;
    this.integrations.add(integration);
    integration.integrated = true;
    LogCodeManager.log("18:00:00", [integration._key, this.agileInstance().key], integration);
    return true;
  }
  update(componentInstance, updatedData) {
    this.integrations.forEach((integration) => {
      if (!integration.ready) {
        LogCodeManager.log("18:02:00", [integration._key]);
        return;
      }
      if (integration.methods.updateMethod)
        integration.methods.updateMethod(componentInstance, updatedData);
    });
  }
  hasIntegration() {
    return this.integrations.size > 0;
  }
};
let Integrations = _Integrations;
Integrations.initialIntegrations = [];

export { Integrations };
