import { RuntimeJob } from './runtime.job.js';
import { defineConfig, generateId } from '@agile-ts/utils';
import { LogCodeManager } from '../logCodeManager.js';

class Observer {
  constructor(agileInstance, config = {}) {
    this.dependents = new Set();
    this.subscribedTo = new Set();
    config = defineConfig(config, {
      dependents: [],
      subs: []
    });
    this.agileInstance = () => agileInstance;
    this._key = config.key;
    this.value = config.value;
    this.previousValue = config.value;
    config.dependents?.forEach((observer) => this.addDependent(observer));
    config.subs?.forEach((subscriptionContainer) => subscriptionContainer.addSubscription(this));
  }
  set key(value) {
    this._key = value;
  }
  get key() {
    return this._key;
  }
  ingest(config = {}) {
    config = defineConfig(config, {
      perform: true,
      background: false,
      sideEffects: {
        enabled: true,
        exclude: []
      },
      force: false
    });
    const job = new RuntimeJob(this, {
      force: config.force,
      sideEffects: config.sideEffects,
      background: config.background,
      key: config.key ?? `${this._key != null ? this._key + "_" : ""}${generateId()}`
    });
    this.agileInstance().runtime.ingest(job, {
      perform: config.perform
    });
  }
  perform(job) {
    LogCodeManager.log("17:03:00");
  }
  addDependent(observer) {
    if (!this.dependents.has(observer))
      this.dependents.add(observer);
  }
  removeDependent(observer) {
    if (this.dependents.has(observer))
      this.dependents.delete(observer);
  }
}

export { Observer };
