import { defineConfig, generateId, isValidObject } from '@agile-ts/utils';

class SubscriptionContainer {
  constructor(subs, config = {}) {
    this.ready = false;
    this.updatedSubscribers = new Set();
    this.isObjectBased = false;
    config = defineConfig(config, {
      key: generateId()
    });
    this.subscribers = new Set();
    this.key = config.key;
    this.componentId = config?.componentId;
    this.subscriberKeysWeakMap = new WeakMap();
    this.selectorsWeakMap = new WeakMap();
    this.isObjectBased = !Array.isArray(subs);
    for (const key in subs) {
      this.addSubscription(subs[key], {
        proxyPaths: config.proxyWeakMap?.get(subs[key])?.paths,
        selectorMethods: config.selectorWeakMap?.get(subs[key])?.methods,
        key: this.isObjectBased ? key : void 0
      });
    }
  }
  addSubscription(sub, config = {}) {
    const toAddSelectorMethods = config.selectorMethods ?? [];
    const proxyPaths = config.proxyPaths ?? [];
    for (const path of proxyPaths) {
      toAddSelectorMethods.push((value) => {
        let _value = value;
        for (const branch of path) {
          if (!isValidObject(_value, true))
            break;
          _value = _value[branch];
        }
        return _value;
      });
    }
    const existingSelectorMethods = this.selectorsWeakMap.get(sub)?.methods ?? [];
    const newSelectorMethods = existingSelectorMethods.concat(toAddSelectorMethods);
    if (newSelectorMethods.length > 0)
      this.selectorsWeakMap.set(sub, { methods: newSelectorMethods });
    if (config.key != null)
      this.subscriberKeysWeakMap.set(sub, config.key);
    this.subscribers.add(sub);
    sub.subscribedTo.add(this);
  }
  removeSubscription(sub) {
    if (this.subscribers.has(sub)) {
      this.selectorsWeakMap.delete(sub);
      this.subscriberKeysWeakMap.delete(sub);
      this.subscribers.delete(sub);
      sub.subscribedTo.delete(this);
    }
  }
}

export { SubscriptionContainer };
