import { Persistent } from '../storages/persistent.js';
import { defineConfig } from '@agile-ts/utils';
import { getStorageManager } from '../storages/index.js';

const _StatePersistent = class extends Persistent {
  constructor(state, config = {}) {
    super(state.agileInstance(), {
      loadValue: false
    });
    config = defineConfig(config, {
      loadValue: true,
      storageKeys: [],
      defaultStorageKey: null
    });
    this.state = () => state;
    this.instantiatePersistent({
      key: config.key,
      storageKeys: config.storageKeys,
      defaultStorageKey: config.defaultStorageKey
    });
    if (this.ready && config.loadValue)
      this.initialLoading();
  }
  async initialLoading() {
    super.initialLoading().then(() => {
      this.state().isPersisted = true;
    });
  }
  async loadPersistedValue(storageItemKey) {
    if (!this.ready)
      return false;
    const _storageItemKey = storageItemKey ?? this._key;
    const loadedValue = await getStorageManager()?.get(_storageItemKey, this.config.defaultStorageKey);
    if (loadedValue == null)
      return false;
    this.state().set(loadedValue, {
      storage: false,
      overwrite: true
    });
    this.setupSideEffects(_storageItemKey);
    return true;
  }
  async persistValue(storageItemKey) {
    if (!this.ready)
      return false;
    const _storageItemKey = storageItemKey ?? this._key;
    this.setupSideEffects(_storageItemKey);
    this.rebuildStorageSideEffect(this.state(), _storageItemKey);
    this.isPersisted = true;
    return true;
  }
  setupSideEffects(storageItemKey) {
    const _storageItemKey = storageItemKey ?? this._key;
    this.state().addSideEffect(_StatePersistent.storeValueSideEffectKey, (instance, config) => {
      this.rebuildStorageSideEffect(this.state(), _storageItemKey, config);
    }, { weight: 0 });
  }
  async removePersistedValue(storageItemKey) {
    if (!this.ready)
      return false;
    const _storageItemKey = storageItemKey || this._key;
    this.state().removeSideEffect(_StatePersistent.storeValueSideEffectKey);
    getStorageManager()?.remove(_storageItemKey, this.storageKeys);
    this.isPersisted = false;
    return true;
  }
  formatKey(key) {
    if (key == null && this.state()._key)
      return this.state()._key;
    if (key == null)
      return;
    if (this.state()._key == null)
      this.state()._key = key;
    return key;
  }
  rebuildStorageSideEffect(state, storageItemKey, config = {}) {
    if (config["storage"] == null || config.storage) {
      getStorageManager()?.set(storageItemKey, this.state().getPersistableValue(), this.storageKeys);
    }
  }
};
let StatePersistent = _StatePersistent;
StatePersistent.storeValueSideEffectKey = "rebuildStateStorageValue";

export { StatePersistent };
