import { getStorageManager } from './index.js';
import { defineConfig, copy } from '@agile-ts/utils';
import { LogCodeManager } from '../logCodeManager.js';

const _Persistent = class {
  constructor(agileInstance, config = {}) {
    this.ready = false;
    this.isPersisted = false;
    this.storageKeys = [];
    this.agileInstance = () => agileInstance;
    this._key = _Persistent.placeHolderKey;
    config = defineConfig(config, {
      loadValue: true,
      storageKeys: [],
      defaultStorageKey: null
    });
    this.config = { defaultStorageKey: config.defaultStorageKey };
    if (config.loadValue) {
      this.instantiatePersistent({
        storageKeys: config.storageKeys,
        key: config.key,
        defaultStorageKey: config.defaultStorageKey
      });
    }
  }
  set key(value) {
    this.setKey(value);
  }
  get key() {
    return this._key;
  }
  async setKey(value) {
    const oldKey = this._key;
    const wasReady = this.ready;
    if (value === this._key)
      return;
    this._key = value ?? _Persistent.placeHolderKey;
    const isValid = this.validatePersistent();
    if (!wasReady) {
      if (isValid)
        await this.initialLoading();
      return;
    }
    await this.removePersistedValue(oldKey);
    if (isValid)
      await this.persistValue(value);
  }
  instantiatePersistent(config = {}) {
    this._key = this.formatKey(config.key) ?? _Persistent.placeHolderKey;
    this.assignStorageKeys(config.storageKeys, config.defaultStorageKey);
    this.validatePersistent();
    const storageManager = getStorageManager();
    if (this._key !== _Persistent.placeHolderKey && storageManager != null) {
      storageManager.persistentInstances[this._key] = this;
    }
  }
  validatePersistent() {
    let isValid = true;
    if (this._key === _Persistent.placeHolderKey) {
      LogCodeManager.log("12:03:00");
      isValid = false;
    }
    if (this.config.defaultStorageKey == null || this.storageKeys.length <= 0) {
      LogCodeManager.log("12:03:01");
      isValid = false;
    }
    this.storageKeys.map((key) => {
      if (!getStorageManager()?.storages[key]) {
        LogCodeManager.log("12:03:02", [this._key, key]);
        isValid = false;
      }
    });
    this.ready = isValid;
    return isValid;
  }
  assignStorageKeys(storageKeys = [], defaultStorageKey) {
    const _storageKeys = copy(storageKeys);
    if (defaultStorageKey != null && !_storageKeys.includes(defaultStorageKey))
      _storageKeys.push(defaultStorageKey);
    if (_storageKeys.length <= 0) {
      const defaultStorageKey2 = getStorageManager()?.config.defaultStorageKey;
      if (defaultStorageKey2 != null) {
        this.config.defaultStorageKey = defaultStorageKey2;
        _storageKeys.push(getStorageManager()?.config.defaultStorageKey);
      }
    } else {
      this.config.defaultStorageKey = defaultStorageKey ?? _storageKeys[0];
    }
    this.storageKeys = _storageKeys;
  }
  async initialLoading() {
    const success = await this.loadPersistedValue();
    if (this.onLoad)
      this.onLoad(success);
    if (!success)
      await this.persistValue();
  }
  async loadPersistedValue(storageItemKey) {
    LogCodeManager.log("00:03:00", ["loadPersistedValue", "Persistent"]);
    return false;
  }
  async persistValue(storageItemKey) {
    LogCodeManager.log("00:03:00", ["persistValue", "Persistent"]);
    return false;
  }
  async removePersistedValue(storageItemKey) {
    LogCodeManager.log("00:03:00", ["removePersistedValue", "Persistent"]);
    return false;
  }
  formatKey(key) {
    return key;
  }
};
let Persistent = _Persistent;
Persistent.placeHolderKey = "__THIS_IS_A_PLACEHOLDER__";

export { Persistent };
